/* PRO-SCAMS "@@(#) dialer_funcs.c, Rev: 1.3.0.0" */

/* -----							*/
/* dialer_funcs()				       12/06/94 */
/*								*/
/* AUTHOR: William C. Brown					*/
/* DESCRIPTION: Supporting functions.				*/
/*								*/
/*							  ----- */

#include <stdio.h>
#include <stropts.h>
#include <sys/types.h>
#include <signal.h>
#include <X11/Intrinsic.h>
#include <X11/StringDefs.h>
#include <X11/Shell.h>
#include <Xol/OpenLook.h>
#include <Xol/ScrollingL.h>
#include <varargs.h>
#include <stropts.h>
#include <termio.h>
#include "uwdialer.h"

void Call_Quit(Widget W, XtPointer ClientData, XtPointer CallData)
{
	struct termios t;
	extern int Modem_fd;
	extern XtInputId Modem_Input;

	if(Modem_fd != 0) {
		(void)Call_Hangup(NULL, NULL, NULL);
		undial(Modem_fd);
	}
		
        exit(0);
} /* Call_Quit */

void Call_Hangup(Widget W, XtPointer ClientData, XtPointer CallData)
{
	struct termios t;
	extern int Server_Pid;
	extern int Modem_fd;
	extern XtInputId Modem_Input;

	if(Modem_fd != 0) {
		if(Modem_Input != NULL) {
			XtRemoveInput(Modem_Input);
			Modem_Input = NULL;
		}
		tcgetattr(Modem_fd, &t);
		t.c_cflag = 0;
		t.c_lflag = 0;
		t.c_iflag = 0;
		t.c_oflag = 0;
		tcsetattr(Modem_fd, TCSANOW, &t);
		undial(Modem_fd);
		close(Modem_fd);
		Modem_fd = 0;
	}

	if(Server_Pid != 0) {
		if(kill(Server_Pid, 0) == 0)
			(void)kill(Server_Pid, SIGKILL);
	}

	(void)Delete_Lock(DELETE);
	(void)Set_Buttons(False);
	(void)Display_Msg(False, "Ready...");
	(void)Update_Connect_Clock((XtPointer)RESET, NULL);
	(void)Set_System_Buttons(False);
} /* Call_Hangup */

void Call_Manual(Widget W, XtPointer ClientData, XtPointer CallData)
{
	(void)Settings(MANUAL);
}

void Call_Upload(Widget W, XtPointer ClientData, XtPointer CallData)
{
	Xfer_Popup(UPLOAD);
}

void Call_Downld(Widget W, XtPointer ClientData, XtPointer CallData)
{
	Xfer_Popup(DOWNLOAD);
}

void Call_SndBrk(Widget W, XtPointer ClientData, XtPointer CallData)
{
	extern int Modem_fd;

	(void)tcsendbreak(Modem_fd, 0);
}

void Call_Log(Widget W, XtPointer ClientData, XtPointer CallData)
{
	(void)Logger_Popup();
} /* Call_Log */

void Call_Add(Widget W, XtPointer ClientData, XtPointer CallData)
{
	(void)Settings(SYSTEMS);
} /* Call_Add */

XtPointer CONVERT OLARGLIST((w, from_string, to_type, to_size, success))
OLARG(Widget, w)
OLARG(char*, from_string)
OLARG(char*, to_type)
OLARG(int, to_size)
OLGRA(Boolean*, success)
{
    XrmValue		fromVal, toVal;	/* resource holders		*/
    Boolean		convResult;	/* return value			*/
    unsigned char	oByte;		/* one byte result		*/
    unsigned short	tByte;		/* two byte result		*/
    caddr_t		fByte;		/* four byte result		*/
    caddr_t		aByte;		/* allocated result		*/
    
    fByte = aByte = NULL;
    *success = False;

    if( strcmp(XtRString, to_type) == 0 )
    {
	*success = True;
	return( from_string );
    }

    to_size = strlen(from_string);
        
    fromVal.size = strlen(from_string) + 1;
    fromVal.addr = from_string;

    switch( to_size )
    {
    case 1:
	toVal.size = sizeof(unsigned char);
	toVal.addr = (caddr_t)&oByte;
	break;
    case 2:
	toVal.size = sizeof(unsigned short);
	toVal.addr = (caddr_t)&tByte;
	break;
    default:
	toVal.size = sizeof(caddr_t);
	toVal.addr = (caddr_t)&fByte;
	break;
    }
    
    convResult = XtConvertAndStore(w, 
				   XtRString, 
				   &fromVal,
				   to_type,
				   &toVal);
    
    if( convResult )
    {
	switch( to_size )
	{
	case 1:
	    fByte = (caddr_t)oByte;
	    break;
	case 2:
	    fByte = (caddr_t)tByte;
	    break;
	default:
	    break;
	}
    }

    if( !convResult && toVal.size != to_size )
    {
	toVal.addr = (caddr_t)XtMalloc(toVal.size);
	fByte = aByte = toVal.addr;
	convResult = XtConvertAndStore(w, 
				       XtRString, 
				       &fromVal,
				       to_type,
				       &toVal);
    }
    
    if( !convResult )
    {
	XtFree(aByte);
	aByte = NULL;
    }
    
    *success = convResult;
    return( fByte );
}

Pixel GetPixelByName(Widget w, char *colorname)
{
	Display *dpy = XtDisplay(w);
	Colormap cmap = DefaultColormap(dpy, DefaultScreen(dpy));
	XColor color, ignore;

	if(XAllocNamedColor(dpy, cmap, colorname, &color, &ignore))
		return(color.pixel);
	else 
		(void)DialerError(NONFATAL, COLORALLOCATION, colorname);
}

void DialerError(Fatal, va_alist)
int Fatal;
va_dcl
{
	FILE *fopen(), *fp;
	va_list ap;
	char *ptr;
	Boolean NoConsole;

	if((fp = fopen("/dev/conslog", "w")) == NULL)
		NoConsole = True;

	va_start(ap);
	ptr = va_arg(ap, char *);

	(void)fprintf(stderr, "UWDIALER ERROR: ");
	(void)vfprintf(stderr, ptr, ap);

	if(NoConsole == False) {
		(void)fprintf(fp, "UWDIALER ERROR: ");
		(void)vfprintf(fp, ptr, ap);
		(void)fclose(fp);
	}

	va_end(ap);

	if(Fatal == FATAL)
		exit(0);
}

void Init_Xterm()
{
	struct termios new;
	extern struct termios old;
	extern XtAppContext app_con;
	extern int Modem_fd;
	extern XtInputId Term_Input;
	
	tcgetattr(0, &old);
        (void)memcpy(&new, &old, sizeof(struct termios));

        /* Run our terminal in RAW() mode */

        new.c_iflag &= ~(BRKINT | ICRNL | INPCK | ISTRIP| IXON);
        new.c_lflag &= ~(ECHO | ICANON | IEXTEN | ISIG);
        new.c_cflag &= ~(CSIZE | PARENB);
        new.c_oflag &= ~(OPOST);
        new.c_cc[VMIN] = 1;
        new.c_cc[VTIME] = 0;
        tcsetattr(0, TCSANOW, &new);

        Term_Input = XtAppAddInput(app_con, Modem_fd, XtInputReadMask,
                         Chars_At_Term, NULL);

} /* Init_Xterm */

void Chars_At_Term(XtPointer ClientData, int *Src, XtInputId *inid)
{
        char buffer[BUFSIZ];
        int nbytes;
	extern int Modem_fd;
	extern int Xfer_Direction;

        if((nbytes = read(fileno(stdin), buffer, BUFSIZ)) < 0)
                exit(0);
        else
                write(Modem_fd, buffer, nbytes);
} /* Chars_At_Term */

void Set_Buttons(int indicator)
{
	int Sentinel;
	Arg args[1];
	extern Widget Flats;

	XtSetArg(args[0], XtNbusy, indicator);
        for(Sentinel=0;Sentinel<MAIN_BUTTONS;Sentinel++) {
		if((Sentinel == 0) || (Sentinel == 3))
			continue;
        	OlFlatSetValues(Flats, Sentinel, args, 1);
	}
} /* Set_Buttons */

void Set_System_Buttons(int indicator)
{
	Arg args[1];
	extern Widget Flats;

	XtSetArg(args[0], XtNbusy, indicator);
        OlFlatSetValues(Flats, 1, args, 1);
        OlFlatSetValues(Flats, 2, args, 1);
} /* Set_Buttons */
